@@ -201,6 +201,13 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                             route.window.needs_render_after_occlusion = false;
                         }
 
+                        // Mark the renderable content as needing to render
+                        if let Some(ctx_item) =
+                            route.window.screen.ctx_mut().get_mut(route_id)
+                        {
+                            ctx_item.val.renderable_content.pending_update.set_dirty();
+                        }
+
                         // Check if we need to throttle based on timing
                         if let Some(wait_duration) = route.window.wait_until() {
                             // We need to wait before rendering again
@@ -226,13 +233,15 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     }
                 }
             }
-            RioEventType::Rio(RioEvent::TerminalDamaged { route_id, damage }) => {
+
+            RioEventType::Rio(RioEvent::Wakeup(route_id)) => {
                 if self.config.renderer.strategy.is_event_based() {
                     if let Some(route) = self.router.routes.get_mut(&window_id) {
                         // Skip rendering for unfocused windows if configured
                         if self.config.renderer.disable_unfocused_render
                             && !route.window.is_focused
                         {
+                            tracing::trace!("Wakeup: Skipping unfocused window");
                             return;
                         }
 
@@ -241,31 +250,21 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                             && route.window.is_occluded
                             && !route.window.needs_render_after_occlusion
                         {
+                            tracing::trace!("Wakeup: Skipping occluded window");
                             return;
                         }
 
-                        // Check if rendering is actually needed
-                        let terminal =
-                            route.window.screen.ctx_mut().current_mut().terminal.lock();
-                        if !terminal.needs_render() {
-                            // Skip rendering if no actual damage
-                            return;
-                        }
-                        drop(terminal);
-
-                        // Clear the one-time render flag if it was set
-                        if route.window.needs_render_after_occlusion {
-                            route.window.needs_render_after_occlusion = false;
-                        }
+                        tracing::trace!(
+                            "Wakeup: Marking route {} for damage check",
+                            route_id
+                        );
 
+                        // Mark the renderable content as needing to check for damage
+                        // The actual damage retrieval will happen during render
                         if let Some(ctx_item) =
                             route.window.screen.ctx_mut().get_mut(route_id)
                         {
-                            ctx_item
-                                .val
-                                .renderable_content
-                                .pending_update
-                                .invalidate(damage, &ctx_item.val.terminal);
+                            ctx_item.val.renderable_content.pending_update.set_dirty();
                             route.schedule_redraw(&mut self.scheduler, route_id);
                         }
                     }
@@ -396,18 +395,36 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             RioEventType::Rio(RioEvent::CursorBlinkingChangeOnRoute(route_id)) => {
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
                     if route_id == route.window.screen.ctx().current_route() {
-                        // Use the new damage_cursor_blink method for more efficient cursor blinking
-                        let mut terminal =
-                            route.window.screen.ctx_mut().current_mut().terminal.lock();
-                        terminal.damage_cursor_blink();
+                        // Get cursor position for damage
+                        let cursor_line = {
+                            let terminal = route
+                                .window
+                                .screen
+                                .ctx_mut()
+                                .current_mut()
+                                .terminal
+                                .lock();
+                            terminal.cursor().pos.row.0 as usize
+                        };
 
-                        // Only request redraw if there's actual damage
-                        let needs_render = terminal.needs_render();
-                        drop(terminal); // Release lock before requesting redraw
+                        // Set UI damage for cursor line
+                        route
+                            .window
+                            .screen
+                            .ctx_mut()
+                            .current_mut()
+                            .renderable_content
+                            .pending_update
+                            .set_ui_damage(rio_backend::event::TerminalDamage::Partial(
+                                [rio_backend::crosswords::LineDamage::new(
+                                    cursor_line,
+                                    true,
+                                )]
+                                .into_iter()
+                                .collect(),
+                            ));
 
-                        if needs_render {
-                            route.request_redraw();
-                        }
+                        route.request_redraw();
                     }
                 }
             }
