@@ -852,33 +852,60 @@ impl Renderer {
                 continue;
             }
 
-            // Take the pending snapshot
-            let terminal_snapshot =
-                match context.renderable_content.pending_update.take_snapshot() {
-                    Some(snapshot) => snapshot,
-                    None if force_full_damage => {
-                        // Force full damage case - create a fresh snapshot
-                        let mut terminal = context.terminal.lock();
-                        let snapshot = TerminalSnapshot {
-                            colors: terminal.colors,
-                            display_offset: terminal.display_offset(),
-                            blinking_cursor: terminal.blinking_cursor,
-                            visible_rows: terminal.visible_rows(),
-                            cursor: terminal.cursor(),
-                            damage: TerminalDamage::Full,
-                            columns: terminal.columns(),
-                            screen_lines: terminal.screen_lines(),
-                        };
-                        terminal.reset_damage();
-                        drop(terminal);
-                        snapshot
-                    }
-                    None => {
-                        // No pending update and not forcing
-                        continue;
-                    }
+            // Get UI damage before resetting
+            let ui_damage = context.renderable_content.pending_update.take_ui_damage();
+            context.renderable_content.pending_update.reset();
+
+            // Compute snapshot at render time
+            let terminal_snapshot = {
+                let mut terminal = context.terminal.lock();
+
+                // Get damage from terminal
+                let terminal_damage = if force_full_damage {
+                    Some(TerminalDamage::Full)
+                } else {
+                    terminal.peek_damage_event()
                 };
 
+                // Merge terminal damage with UI damage
+                let damage = match (terminal_damage, ui_damage) {
+                    (Some(TerminalDamage::Full), _) | (_, Some(TerminalDamage::Full)) => {
+                        TerminalDamage::Full
+                    }
+                    (Some(term), Some(ui)) => {
+                        // Merge partial damages
+                        match (term, ui) {
+                            (
+                                TerminalDamage::Partial(mut lines1),
+                                TerminalDamage::Partial(lines2),
+                            ) => {
+                                lines1.extend(lines2);
+                                TerminalDamage::Partial(lines1)
+                            }
+                            _ => TerminalDamage::Full,
+                        }
+                    }
+                    (Some(damage), None) => damage,
+                    (None, Some(damage)) => damage,
+                    (None, None) => TerminalDamage::Full,
+                };
+
+                let snapshot = TerminalSnapshot {
+                    colors: terminal.colors,
+                    display_offset: terminal.display_offset(),
+                    blinking_cursor: terminal.blinking_cursor,
+                    visible_rows: terminal.visible_rows(),
+                    cursor: terminal.cursor(),
+                    damage,
+                    columns: terminal.columns(),
+                    screen_lines: terminal.screen_lines(),
+                };
+                terminal.reset_damage();
+                drop(terminal);
+
+                snapshot
+            };
+
             // Get hint matches from renderable content
             let hint_matches = context.renderable_content.hint_matches.as_deref();
 
@@ -949,20 +976,6 @@ impl Renderer {
                             context.renderable_content.is_blinking_cursor_visible =
                                 !context.renderable_content.is_blinking_cursor_visible;
                             context.renderable_content.last_blink_toggle = Some(now);
-
-                            if let Some(ref mut lines) = specific_lines {
-                                lines.insert(LineDamage {
-                                    line: context
-                                        .renderable_content
-                                        .cursor
-                                        .state
-                                        .pos
-                                        .row
-                                        .0
-                                        as usize,
-                                    damaged: true,
-                                });
-                            }
                         }
                     } else {
                         // When not blinking (e.g., during typing), ensure cursor is visible
